import json
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_http_methods
from django_bitbucket_hook.models import Hook


@csrf_exempt
@require_http_methods(["POST"])
def only_hook(request):
    payload, user, repo = get_payload(request)

    if not user and not repo:
        return JsonResponse({'success': False, 'message': 'No JSON data or URL argument : cannot identify hook'})

    try:
        hook = Hook.objects.get(user=user, repo=repo)
        hook.execute()
    except Hook.DoesNotExist:
        return JsonResponse({'success': False, 'message': 'Not exist Hook'})
    return JsonResponse({'success': True})


@csrf_exempt
@require_http_methods(["POST"])
def hook_name(request, name):
    payload, user, repo = get_payload(request)

    if not user and not repo and not name:
        return JsonResponse({'success': False, 'message': 'No JSON data or URL argument : cannot identify hook'})

    try:
        hook = Hook.objects.get(user=user, repo=repo, name=name)
        hook.execute()
    except Hook.DoesNotExist:
        return JsonResponse({'success': False, 'message': 'Not exist Hook'})
    return JsonResponse({'success': True})


@csrf_exempt
@require_http_methods(["POST"])
def hook_name_branch(request, name, branch):
    payload, user, repo = get_payload(request)
    repo_branch = payload.get('ref', False)

    if repo_branch is False:
        repo_branch = payload.get('commits', False)
        if repo_branch is False:
            return JsonResponse({'success': False, 'message': 'Not exist branch'})
        repo_branch = repo_branch[0].get('branch', False)
    else:
        _, repo_branch = repo_branch.rsplit('/', 1)

    if not user and not repo and not name and not branch and not repo_branch:
        return JsonResponse({'success': False, 'message': 'No JSON data or URL argument : cannot identify hook'})

    try:
        if repo_branch != branch:
            return JsonResponse({'success': False, 'message': 'This is not the right branch'})

        hook = Hook.objects.get(user=user, repo=repo, name=name, branch=branch)
        hook.execute()
    except Hook.DoesNotExist:
        return JsonResponse({'success': False, 'message': 'Not exist Hook'})
    return JsonResponse({'success': True})


def get_payload(request):
    # Git repo information from post-receive payload
    if request.META.get('CONTENT_TYPE') == "application/json":
        payload = json.loads(request.body.decode('utf8'))
    else:
        # Probably application/x-www-form-urlencoded
        payload = json.loads(request.POST.get("payload", "{}"))

    repo_data = payload.get('repository', {})

    user = repo_data.get('owner', {})
    repo = repo_data.get('name', None)

    if isinstance(user, dict):
        user = user.get('name', user.get('username', None))

    return payload, user, repo



import os
import json
from django.test import TestCase
from django_bitbucket_hook.models import Hook


class DjangoBitbucketHookTestCase(TestCase):
    def setUp(self):
        Hook.objects.create(
            name='test-name',
            user='user',
            repo='repo',
            path='date',  # Put absolute path to executable file. Example: /home/test/test.sh..
            branch='master'
        )

        self.bitbucket_payload = """
            {
              "ref": "refs/heads/master",
                  "repository": {
                    "id": "000000000",
                    "name": "repo",
                    "full_name": "user/repo",
                    "owner": {
                      "name": "user",
                      "email": "user@email"
                    }
                  }
            }
        """

    def tearDown(self):
        if os.path.isfile("test.txt"):
            os.remove('test.txt')

    def test_broke__main_url(self):
        response = self.client.post('/')
        content = json.loads(response.content.decode('utf-8'))
        self.assertFalse(content['success'])
        self.assertEqual(content['message'], 'No JSON data or URL argument : cannot identify hook')
        self.assertEqual(response.status_code, 200)

    def test_broke_repo_url(self):
        response = self.client.post('/test')
        content = json.loads(response.content.decode('utf-8'))
        self.assertFalse(content['success'])
        self.assertEqual(content['message'], 'Not exist Hook')
        self.assertEqual(response.status_code, 200)

    def test_broke_repo_branch_url(self):
        response = self.client.post('/test/test')
        content = json.loads(response.content.decode('utf-8'))
        self.assertFalse(content['success'])
        self.assertEqual(content['message'], 'Not exist branch')
        self.assertEqual(response.status_code, 200)

    def test_main_url(self):
        data = {"payload": self.bitbucket_payload}
        response = self.client.post('/', data)
        content = json.loads(response.content.decode('utf-8'))
        self.assertIsInstance(content, dict)
        self.assertEqual(response.status_code, 200)

    def test_repo_url(self):
        data = {"payload": self.bitbucket_payload}
        response = self.client.post('/test-name', data)
        content = json.loads(response.content.decode('utf-8'))
        self.assertIsInstance(content, dict)
        self.assertTrue(content['success'])
        self.assertEqual(response.status_code, 200)

    def test_repo_broke_branch_url(self):
        data = {"payload": self.bitbucket_payload}
        response = self.client.post('/test-name/not-right', data)
        content = json.loads(response.content.decode('utf-8'))
        self.assertIsInstance(content, dict)
        self.assertFalse(content['success'])
        self.assertEqual(content['message'], 'This is not the right branch')
        self.assertEqual(response.status_code, 200)

    def test_repo_branch_url(self):
        data = {"payload": self.bitbucket_payload}
        response = self.client.post('/test-name/master', data)
        content = json.loads(response.content.decode('utf-8'))
        self.assertIsInstance(content, dict)
        self.assertTrue(content['success'])
        self.assertEqual(response.status_code, 200)



from django.conf.urls import patterns, url
from django_bitbucket_hook import views

urlpatterns = patterns(
    '',
    url(r'^$', views.only_hook),
    url(r'^(?P<name>[\w-]+)$', views.hook_name),
    url(r'^(?P<name>[\w-]+)/(?P<branch>[\w-]+)$', views.hook_name_branch),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Hook',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('name', models.CharField(max_length=255)),
                ('user', models.CharField(max_length=255)),
                ('repo', models.CharField(max_length=255)),
                ('path', models.CharField(max_length=255)),
                ('branch', models.CharField(max_length=255)),
            ],
        ),
    ]



from setuptools import setup
long_description = open('README.rst').read()

setup(
    name='django-bitbucket-hook',
    version='1.0.3',
    packages=['django_bitbucket_hook', 'django_bitbucket_hook.migrations'],
    url='https://github.com/Kras4ooo/django-bitbucket-hook',
    license='Apache License Version 2.0',
    author='Krasimir',
    author_email='krasimir.nikolov1994@gmail.com',
    long_description=long_description,
    classifiers=[
        'Intended Audience :: Developers',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    description='Django module',
    test_suite='tests'
)



from django.db import models
from django.utils.encoding import python_2_unicode_compatible
import subprocess


@python_2_unicode_compatible
class Hook(models.Model):
    name = models.CharField(max_length=255)
    user = models.CharField(max_length=255)
    repo = models.CharField(max_length=255)
    path = models.CharField(max_length=255)
    branch = models.CharField(max_length=255)

    def execute(self):
        subprocess.call([self.path.encode('utf-8')])

    def __str__(self):
        return "%s (%s/%s)" % (self.name, self.user, self.repo)
